/* ZM - Z's Mathematics Toolbox
 * (C)Copyright Zhidao since 1998, all rights are reserved.
 */

/*!
 * \mainpage

 ZM is a collection of numerical computation and optimization
 tools including:
 - random number generator (Mersenne twister)
 - statistics
 - special functions (Gauss's error, gamma, beta, Bessel)
 - vectors and matrices of real numbers and complex numbers
 - complex number (arithmetics, quadratic/cubic equation solvers)
 - polynomial expression and equation solver
 - linear equation solvers (generalized, tridiagonal, Lyapunov)
 - matrix decompositions (LU, Choresky, LQ/QR)
 - eigenvalue analysis
 - Fast Fourier Transform
 - graph search and rapidly-explored random tree.
 - kd-tree
 - vector sequence and interpolations (linear, Lagrange, spline
   Akima, polynomial fitting, NURBS)
 - numerical integrator
 - optimization tools (line search, linear programming, linear
   complementary programming, quadratic programming, nonlinear
   programming based on descent/non-descent methods)
 - nonlinear equation solvers
 - first/second order ordinary differential equation quadratures
   (Runge-Kutta family, embedded Runge-Kutta family, Adams, Gear,
   backward Euler, Butcher-Kuntzmann family, deferred correction,
   Leapflog)
 - multiple classification analysis (clustering)
 - nonlinear oscillator
 - parser of mathematical expression
 */

#ifndef __ZM_H__
#define __ZM_H__

#include <zm/zm_sf.h>
#include <zm/zm_mca.h>
#include <zm/zm_pex.h>
#include <zm/zm_nle.h>
#include <zm/zm_ip.h>
#include <zm/zm_seq.h>
#include <zm/zm_opt.h>
#include <zm/zm_ode.h>
#include <zm/zm_intg.h>
#include <zm/zm_oscil.h>

#endif /* __ZM_H__ */
